classdef PointSourceColocForJosephCabralProcess < MaskProcessingProcess
    % for puncta in nuclei, given a reference puncta channel, counts puncta
    % in other channels that fall within certain distance;
    % Modified from SimplePointSourceScoringProcess;
    % Marcelo Cicconet, Apr 2017
    
    methods(Access = public)        
        function obj = PointSourceColocForJosephCabralProcess(owner,varargin)
            
            if nargin == 0
                super_args = {};
            else
                % Input check
                ip = inputParser;
                ip.addRequired('owner',@(x) isa(x,'MovieData'));
                ip.addOptional('outputDir',owner.outputDirectory_,@ischar);
                ip.addOptional('funParams',[],@isstruct);
                ip.parse(owner,varargin{:});
                outputDir = ip.Results.outputDir;
                funParams = ip.Results.funParams;
                
                % Define arguments for superclass constructor
                super_args{1} = owner;
                super_args{2} = PointSourceColocForJosephCabralProcess.getName;
                super_args{3} = @pointSourceColocForJosephCabral;
                if isempty(funParams)                                       
                    funParams = PointSourceColocForJosephCabralProcess.getDefaultParams(owner,outputDir);                               
                end
                super_args{4} = funParams;                    
            end
            
            obj = obj@MaskProcessingProcess(super_args{:});
        end                  
    end
    methods(Static)
        function name =getName()
            name = 'Point Source Colocalization';
        end
        function h = GUI()
            h= @PointSourceColocForJosephCabralProcessGUI;
        end
        
        function funParams = getDefaultParams(owner,varargin)
            % Input check
            ip=inputParser;
            ip.addRequired('owner',@(x) isa(x,'MovieData'));
            ip.addOptional('outputDir',owner.outputDirectory_,@ischar);
            ip.parse(owner, varargin{:});
            outputDir=ip.Results.outputDir;
            
            % Define default process parameters
            funParams.ChannelIndex = [];
            funParams.ProximityThreshold = 1;
            funParams.OutputDirectory = [outputDir  filesep 'point_source_coloc'];
            funParams.BatchMode = false;
        end
    end
end
    