clear, clc
% set paths

pathToTemplateProcessedImage = '/home/mc457/Downloads/JosephCabral/1hpi/1hpi.mat';
pathToFolderToProcess = '/home/mc457/Downloads/JosephCabral/TestBatchProcessing'; % folder containing '.nd2' files

% --------------------------------------------------
% processing

% get batch processing parameters

MD = MovieData.load(pathToTemplateProcessedImage,0);

pkg = MD.packages_{1};
ps = cell(1,5);
for i = 1:5
    p = pkg.processes_{i}.getParameters;
    ps{i} = p;
end

% batch processing

fplist = listfiles(pathToFolderToProcess, '.nd2');

for i = 1:length(fplist)
    fp = fplist{i};
    if strcmp(fp(end-2:end),'nd2')
        fprintf('Processing image...\n%s\n', fp);
        
        % load image
        MD = bfImport(fp);

        %

        disp('--------------------------------------------------')
        disp('1. Segmentation')
        disp('--------------------------------------------------')
        while length(MD.processes_) > 0
            MD.deleteProcess(1)
        end
        MD.addProcess(UnsupervisedSegmentationProcess(MD));
        p = ps{1};
        q = MD.processes_{1}.getParameters;
        q.BatchMode = 1;
        q.ChannelIndex = p.ChannelIndex;
        q.hmin = p.hmin;
        q.sigma = p.sigma;
        MD.processes_{1}.setParameters(q);
        MD.processes_{1}.run

        %

        disp('--------------------------------------------------')
        disp('2. Mask Properties')
        disp('--------------------------------------------------')
        while length(MD.processes_) > 1
            MD.deleteProcess(2)
        end
        MD.addProcess(CalcMaskPropsProcess(MD));
        p = ps{2};
        q = MD.processes_{2}.getParameters;
        q.ChannelIndex = p.ChannelIndex;
        q.selectedProps = p.selectedProps;
        q.saveresultsas = p.saveresultsas;
        q.MaskProcessIndex = p.MaskProcessIndex;
        q.ProcessIndex = p.ProcessIndex;
        q.thresholdableProps = p.thresholdableProps;
        q.maskUsed = p.maskUsed;
        q.newMinMaxRange = p.newMinMaxRange;
        q.BatchMode = 1;
        MD.processes_{2}.setParameters(q);
        MD.processes_{2}.run

        %

        disp('--------------------------------------------------')
        disp('3. Filter Masks')
        disp('--------------------------------------------------')
        while length(MD.processes_) > 2
            MD.deleteProcess(3)
        end
        MD.addProcess(FilterMaskPropsProcess(MD));
        p = ps{3};
        q = MD.processes_{3}.getParameters;
        q.ChannelIndex = p.ChannelIndex;
        q.ProcessIndex = p.ProcessIndex;
        q.thresholdableProps = p.thresholdableProps;
        q.rangeSettingMatrix = p.rangeSettingMatrix;
        q.selectedProps = p.selectedProps;
        q.BatchMode = 1;
        MD.processes_{3}.setParameters(q);
        MD.processes_{3}.run

        %

        disp('--------------------------------------------------')
        disp('4. Point Sources')
        disp('--------------------------------------------------')
        while length(MD.processes_) > 3
            MD.deleteProcess(4)
        end
        MD.addProcess(LoGPointSourceDetectionProcess(MD));
        p = ps{4};
        q = MD.processes_{4}.getParameters;
        q.ChannelIndex = p.ChannelIndex;
        q.globalSigma = p.globalSigma;
        q.globalAlpha = p.globalAlpha;
        q.BatchMode = 1;
        MD.processes_{4}.setParameters(q);
        MD.processes_{4}.run

        %

        disp('--------------------------------------------------')
        disp('5. Scoring')
        disp('--------------------------------------------------')
        while length(MD.processes_) > 4
            MD.deleteProcess(5)
        end
        MD.addProcess(PointSourceColocForJosephCabralProcess(MD));
        p = ps{5};
        q = MD.processes_{5}.getParameters;
        q.ChannelIndex = p.ChannelIndex;
        q.ProximityThreshold = p.ProximityThreshold;
        q.BatchMode = 1;
        MD.processes_{5}.setParameters(q);
        MD.processes_{5}.run

        %

        MD.save
        
        [parentFolder,fileName] = fileparts(fp);
        pscScoresPath = [q.OutputDirectory filesep 'scores.csv'];
        copyfile(pscScoresPath,[parentFolder filesep fileName '_scores.csv']);
    end
end