classdef SimplePointSourceScoringProcess < MaskProcessingProcess
    % A concrete process to score point sources according to masks
    % e.g.: count number of point sources (puncta) per nuclei
    % (Modified from PointSourceScoringProcess. Simpler because doesn't involve region growing/shrinking.)
    % Marcelo Cicconet, Mar 2017
    
    methods(Access = public)        
        function obj = SimplePointSourceScoringProcess(owner,varargin)
            
            if nargin == 0
                super_args = {};
            else
                % Input check
                ip = inputParser;
                ip.addRequired('owner',@(x) isa(x,'MovieData'));
                ip.addOptional('outputDir',owner.outputDirectory_,@ischar);
                ip.addOptional('funParams',[],@isstruct);
                ip.parse(owner,varargin{:});
                outputDir = ip.Results.outputDir;
                funParams = ip.Results.funParams;
                
                % Define arguments for superclass constructor
                super_args{1} = owner;
                super_args{2} = SimplePointSourceScoringProcess.getName;
                super_args{3} = @simplePointSourceScoring;
                if isempty(funParams)                                       
                    funParams = SimplePointSourceScoringProcess.getDefaultParams(owner,outputDir);                               
                end
                super_args{4} = funParams;                    
            end
            
            obj = obj@MaskProcessingProcess(super_args{:});
        end                  
    end
    methods(Static)
        function name =getName()
            name = 'Point Source Scoring';
        end
        function h = GUI()
            h= @simplePointSourceScoringProcessGUI;
        end
        
        function funParams = getDefaultParams(owner,varargin)
            % Input check
            ip=inputParser;
            ip.addRequired('owner',@(x) isa(x,'MovieData'));
            ip.addOptional('outputDir',owner.outputDirectory_,@ischar);
            ip.parse(owner, varargin{:});
            outputDir=ip.Results.outputDir;
            
            % Define default process parameters
            funParams.ChannelIndex = 2;
            funParams.OutputDirectory = [outputDir  filesep 'point_source_scores'];
            funParams.BatchMode = false;
%             funParams.MaskExpansion = 10; % pixels
%             funParams.MaskExpansionFactor = 1.5; % value (> 1) to be multiplied to radius to determine expansion
%             funParams.ExpansionIsConstant = true;
        end
    end
end
    