classdef UnsupervisedSegmentationProcess < SegmentationProcess
    % A process for nuclear segmentation
    % Marcelo Cicconet, March 2017
    
    methods (Access = public)
        function obj = UnsupervisedSegmentationProcess(owner,varargin)
            
            if nargin == 0
                super_args = {};
            else
                % Input check
                ip = inputParser;
                ip.addRequired('owner',@(x) isa(x,'MovieData'));
                ip.addOptional('outputDir',owner.outputDirectory_,@ischar);
                ip.addOptional('funParams',[],@isstruct);
                ip.parse(owner,varargin{:});
                outputDir = ip.Results.outputDir;
                funParams = ip.Results.funParams;
                
                % Define arguments for superclass constructor
                super_args{1} = owner;
                super_args{2} = UnsupervisedSegmentationProcess.getName;
                super_args{3} = @unsupervisedSegmentation;
                if isempty(funParams)
                    funParams = UnsupervisedSegmentationProcess.getDefaultParams(owner,outputDir);
                end
                super_args{4} = funParams;
            end
            
            obj = obj@SegmentationProcess(super_args{:});
        end
        
    end
    methods (Static)
        function name = getName()
            name = 'Unsupervised Segmentation';
        end
        function h = GUI()
            h= @unsupervisedSegmentationProcessGUI;
        end
        function funParams = getDefaultParams(owner,varargin)
            % Input check
            ip=inputParser;
            ip.addRequired('owner',@(x) isa(x,'MovieData'));
            ip.addOptional('outputDir',owner.outputDirectory_,@ischar);
            ip.parse(owner, varargin{:})
            outputDir=ip.Results.outputDir;
            
            % Set default parameters
            funParams.BatchMode = false;
            funParams.ChannelIndex = 1;%:numel(owner.channels_);
            funParams.OutputDirectory = [outputDir  filesep 'SegmentationMasks'];
            funParams.hmin = 0.1;
            funParams.sigma = 2;
        end
    end
end