function logPointSourceDetectionOnMovieData(movieData)
% 
% Marcelo Cicconet, Sep 2016
%

%% ----------- Input --------- %%

ip = inputParser;
ip.addRequired('movieData',@(x)(isa(x,'MovieData')));
ip.parse(movieData);

iProc = movieData.getProcessIndex('LoGPointSourceDetectionProcess',1,0);

%If the process doesn't exist, create it
if isempty(iProc)
    error('No LoGPointSourceDetectionProcess in input movieData! please create the process and use the process.run method to run this function!')
end

proc = movieData.processes_{iProc};

%Parse input, store in parameter structure
p = parseProcessParams(proc);

% original segmentation mask
maskProcessIndex = movieData.getProcessIndex('UnsupervisedSegmentationProcess',Inf,0);
if isempty(maskProcessIndex)
    error('This function requires that the input movie has already been segmented and that a valid UnsupervisedSegmentationProcess be specified!')
end
for i = 1:length(movieData.processes_{maskProcessIndex}.outFilePaths_)
    if ~isempty(movieData.processes_{maskProcessIndex}.outFilePaths_{i})
        maskChan = i;
        break;
    end
end
% maskChan = 1;
inMaskDir = movieData.processes_{maskProcessIndex}.outFilePaths_{maskChan};
inMaskPath = [inMaskDir filesep 'mask_1.tif'];
maskIn = imread(inMaskPath);
nucleiMask = maskIn > 0;

%% ----------- Init ----------- %%

mkClrDir(p.OutputDirectory)

outFilePaths = cell(1,numel(movieData.channels_));

nChan = length(p.ChannelIndex);
for j = 1:nChan
    outFilePaths{1,p.ChannelIndex(j)} = [p.OutputDirectory filesep 'channel_' num2str(p.ChannelIndex(j)) '.mat'];   
end
proc.setOutFilePaths(outFilePaths);


%% ---------------- Point Source Detection --------------- %%

if ~p.BatchMode && feature('ShowFigureWindows')
    displayProgressBar = 1;
else
    displayProgressBar = 0;
end

nChan = length(p.ChannelIndex);
for j = 1:nChan
    I = movieData.channels_(p.ChannelIndex(j)).loadImage(1);
    fprintf('detecting point sources in channel %d\n', p.ChannelIndex(j));
    
%     if p.extrapolateSignalNoise == false
%         ptSrcImg  = blobRobustPointSourceDetection(I, p.globalSigma, displayProgressBar, j, nChan, 'Alpha', p.globalAlpha);
%     else
%         if isa(I, 'integer')
%             I = double(I);
%         else
%             error('image not of type integer; model for extrapolated signal noise might not be correct');
%         end
%         I = I+rand(size(I));
%         ptSrcImg  = blobRobustPointSourceDetection(I, p.globalSigma, displayProgressBar, j, nChan, 'Alpha', p.globalAlpha);
%     end
    if length(p.globalAlpha) == 1
        ptSrcImg = logPointSourceDetection(I,nucleiMask,p.globalSigma,p.globalAlpha);
        fprintf('using threshold %d for channel %d\n', p.globalAlpha, p.ChannelIndex(j));
    elseif length(p.globalAlpha) == length(p.ChannelIndex)
        ptSrcImg = logPointSourceDetection(I,nucleiMask,p.globalSigma,p.globalAlpha(j));
        fprintf('using threshold %d for channel %d\n', p.globalAlpha(j), p.ChannelIndex(j));
    end
    
    idx = find(ptSrcImg);
    [yCoord,xCoord] = ind2sub(size(ptSrcImg),idx);
    amp = ptSrcImg(idx);
    
%     figure
%     imshow(I), hold on
%     plot(xCoord(~isnan(amp)),yCoord(~isnan(amp)),'or')
%     plot(xCoord(isnan(amp)),yCoord(isnan(amp)),'og')
%     hold off
%     pause
    
    clear movieInfo
    movieInfo = struct('xCoord', xCoord, 'yCoord', yCoord, 'amp', amp, 'sigmaX', [], 'sigmaY', [], 'bkg', []);
    save(outFilePaths{1,p.ChannelIndex(j)}, 'movieInfo');
end



%% ------ Finalize ------ %%


%Store parameters/settings in movieData structure
movieData.processes_{iProc}.setDateTime;
movieData.save; %Save the new movieData to disk

disp('Finished detecting point sources!')

