classdef CalcMaskPropsProcess < ImageAnalysisProcess
    % A concrete class for calculating properties for objects in a labled mask or a binary mask
    % input: moviedata's labled mask output
    % output: for each mask, output a mat for the object MaskPropsCalc all properties.
    % optional output: for each mask, and for each property, write a csv named
    % as the property name.
    methods (Access = public)
        function obj = CalcMaskPropsProcess(owner, varargin)
            % Input check
            ip = inputParser;
            ip.addRequired('owner',@(x) isa(x,'MovieData'));
            ip.addOptional('outputDir',owner.outputDirectory_,@ischar);
            ip.addOptional('funParams',[],@isstruct);
            ip.parse(owner,varargin{:});
            outputDir = ip.Results.outputDir;
            funParams = ip.Results.funParams;
            
            % Constructor of the CalcMaskPropsProcess
            super_args{1} = owner;
            super_args{2} = CalcMaskPropsProcess.getName;
            super_args{3} = @calcMovieMaskProperties;
            if isempty(funParams)  % Default funParams
                funParams = CalcMaskPropsProcess.getDefaultParams(owner,outputDir);
            end
            super_args{4} = funParams;
            obj = obj@ImageAnalysisProcess(super_args{:});
        end
        
        function varargout = loadChannelOutput(obj,iChan,varargin)
            % Input check
            outputList = {'MPropsC'};
            ip =inputParser;
            ip.addRequired('iChan',@(x) isscalar(x) && obj.checkChanNum(x));
            ip.addOptional('iFrame',1:obj.owner_.nFrames_,@(x) all(obj.checkFrameNum(x)));
            ip.addParamValue('output',outputList,@(x) all(ismember(x,outputList)));
            ip.parse(iChan,varargin{:})
            iFrame = ip.Results.iFrame;
            output = ip.Results.output;
            if ischar(output),output={output}; end
            
            % Data loading
            s = load(obj.outFilePaths_{1,iChan},output{:});
            
            if numel(ip.Results.iFrame)>1,
                varargout{1}=s.(output{1});
            else
                varargout{1}=s.(output{1})(iFrame);
            end
        end
        function output = getDrawableOutput(obj)
            %colors = hsv(numel(obj.owner_.channels_));
            output(1).name='Properties';
            output(1).var='MPropsC';
            output(1).formatData=@CalcMaskPropsProcess.formatOutput;
            output(1).type='overlay';
            output(1).defaultDisplayMethod=@TextDisplay;
        end
        
        
    end
    methods(Static)
        function name = getName()
            name = 'Mask Property Calculations';
        end
        function h = GUI()
            h = @calcMaskPropsGUI;
        end
        function funParams = getDefaultParams(owner,varargin)
            % Input check
            ip=inputParser;
            ip.addRequired('owner',@(x) isa(x,'MovieData'));
            ip.addOptional('outputDir',owner.outputDirectory_,@ischar);
            ip.parse(owner, varargin{:})
            outputDir=ip.Results.outputDir;
            
            % Set default parameters
            funParams.ChannelIndex = 1:numel(owner.channels_);
            funParams.OutputDirectory = [outputDir  filesep 'masksPropsCalc'];
            funParams.ProcessIndex = [];%Default is to use raw images
            funParams.MaskProcessIndex = 1;
            funParams.saveresultsas = 1;
        end
        function y =formatOutput(x)
            % Format output in xy coordinate system
            if isempty(x.xCoord)
                y = NaN(1,2);
            else
                y = horzcat(x.xCoord(:,1),x.yCoord(:,1));
            end
        end
    end
end
