classdef FilterMaskPropsProcess < MaskProcessingProcess
    % A concrete class for calculating properties for objects in a labled mask or a binary mask
    % input: moviedata's labled mask output
    % output: for each mask, output a mat for the object MaskPropsCalc all properties.
    % optional output: for each mask, and for each property, write a csv named
    % as the property name.
    methods (Access = public)
        function obj = FilterMaskPropsProcess(owner, varargin)
            % Input check
            ip = inputParser;
            ip.addRequired('owner',@(x) isa(x,'MovieData'));
            ip.addOptional('outputDir',owner.outputDirectory_,@ischar);
            ip.addOptional('funParams',[],@isstruct);
            ip.addOptional('funName',[],@ischar);
            ip.parse(owner,varargin{:});
            outputDir = ip.Results.outputDir;
            funParams = ip.Results.funParams;
            funName = ip.Results.funName;
            % Constructor of the CalcMaskPropsProcess
            super_args{1} = owner;
            if isempty(funName)
            super_args{2} = FilterMaskPropsProcess.getName;
            else
                super_args{2} = funName;
            end
            super_args{3} = @filterMovieMaskProperties;
            if isempty(funParams)  % Default funParams
                funParams = FilterMaskPropsProcess.getDefaultParams(owner,outputDir);
            end
            super_args{4} = funParams;
            obj = obj@MaskProcessingProcess(super_args{:});
        end
        
%         function varargout = loadChannelOutput(obj,iChan,varargin)
%             % Input check
%             outputList = {'movieInfo'};
%             ip =inputParser;
%             ip.addRequired('iChan',@(x) isscalar(x) && obj.checkChanNum(x));
%             ip.addOptional('iFrame',1:obj.owner_.nFrames_,@(x) all(obj.checkFrameNum(x)));
%             ip.addParamValue('output',outputList,@(x) all(ismember(x,outputList)));
%             ip.parse(iChan,varargin{:})
%             iFrame = ip.Results.iFrame;
%             output = ip.Results.output;
%             if ischar(output),output={output}; end
%             
%             % Data loading
%             s = load(obj.outFilePaths_{1,iChan},output{:});
%             
%             if numel(ip.Results.iFrame)>1,
%                 varargout{1}=s.(output{1});
%             else
%                 varargout{1}=s.(output{1})(iFrame);
%             end
%         end

    end
    methods(Static)
        function name = getName()
            name = 'Mask Property Filtering';
        end
        function h = GUI()
            h = @filterMaskPropsGUI;
        end
        function funParams = getDefaultParams(owner,varargin)
            % Input check
            ip=inputParser;
            ip.addRequired('owner',@(x) isa(x,'MovieData'));
            ip.addOptional('outputDir',owner.outputDirectory_,@ischar);
            ip.parse(owner, varargin{:})
            outputDir=ip.Results.outputDir;
            
            % Set default parameters
            funParams.ChannelIndex = 1:numel(owner.channels_);
            funParams.OutputDirectory = [outputDir  filesep 'masksPropsFilter'];
            funParams.ProcessIndex = [];%Default is to use raw images
        end
    end
    methods
        function run(obj,varargin)
            obj.resetDisplayMethod();
            obj.success_=false;
            
            % Run the process!
            %obj.startTime_ = clock;
            obj.funName_(obj.getOwner(), obj);
            
            % Update flags and set finishTime
            obj.success_= true;
            obj.updated_= true;
            obj.procChanged_= false;
            %obj.finishTime_ = clock;
            
            % Run sanityCheck on parent package to update dependencies
            for packId = obj.getPackageIndex()
                obj.getOwner().getPackage(packId).sanityCheck(false,'all');
            end
            
            obj.getOwner().save();
            %obj.run(obj, obj);
        end
    end
    
end
