classdef MaskObjectPairingProcess < MaskProcessingProcess
% MASKOBJECTPAIRINGPROCESS is a mask processing process that takes in two
% sets of masks and pair them to each other according to distances or other
% expandable methods.
    methods (Access = public)
        function obj = MaskObjectPairingProcess(owner, varargin)
            % Input check
            ip = inputParser;
            ip.addRequired('owner',@(x) isa(x,'MovieData'));
            ip.addOptional('outputDir',owner.outputDirectory_,@ischar);
            ip.addOptional('funParams',[],@isstruct);
            %ip.addOptional('funName',[],@ischar);
            ip.parse(owner,varargin{:});
            outputDir = ip.Results.outputDir;
            funParams = ip.Results.funParams;
            %funName = ip.Results.funName;
            super_args{1} = owner;
            %if isempty(funName)
            super_args{2} = MaskObjectPairingProcess.getName;
            %else
            %    super_args{2} = funName;
            %end
            super_args{3} = @pairingMovieObjects;
            if isempty(funParams)  % Default funParams
                funParams = MaskObjectPairingProcess.getDefaultParams(owner,outputDir);
            end
            super_args{4} = funParams;
            obj = obj@MaskProcessingProcess(super_args{:});
        end
        
        function varargout = loadChannelMatOutput(obj,iChan,varargin)
            % Input check
            outputList = {'ParingTable'};
            ip =inputParser;
            ip.addRequired('iChan',@(x) isscalar(x) && obj.checkChanNum(x));
            ip.addOptional('iFrame',1:obj.owner_.nFrames_,@(x) all(obj.checkFrameNum(x)));
            ip.addParamValue('output',outputList,@(x) all(ismember(x,outputList)));
            ip.parse(iChan,varargin{:})
            iFrame = ip.Results.iFrame;
            output = ip.Results.output;
            if ischar(output),output={output}; end
            
            % Data loading
            s = load(obj.outFilePaths_{1,iChan},output{:});
            
            if numel(ip.Results.iFrame)>1,
                varargout{1}=s.(output{1});
            else
                varargout{1}=s.(output{1})(iFrame);
            end
        end
        
    end
    methods(Static)
        function name = getName()
            name = 'Mask Object Pairing';
        end
        function h = GUI()
            h = @maskParingGUI;
        end
        function funParams = getDefaultParams(owner,varargin)
            % Input check
            ip=inputParser;
            ip.addRequired('owner',@(x) isa(x,'MovieData'));
            ip.addOptional('outputDir',owner.outputDirectory_,@ischar);
            ip.parse(owner, varargin{:})
            outputDir=ip.Results.outputDir;
            
            % Set default parameters
            funParams.ChannelIndex = 1:numel(owner.channels_);
            funParams.OutputDirectory = [outputDir  filesep 'MaskObjectParing'];
            funParams.ProcessIndex = [];%Default is to use raw 
            funParams.methodInd = 1;
            funParams.distanceThreshold = 50;
        end
    end
end