classdef MaskPropsCalc < hgsetget %MaskPropsCalc
    %defines the list of property calculations needed for the objects in a
    %mask or matrix
    properties (SetAccess = private, GetAccess = public)
        PropertyList
        ListSwitch
        FunctionList
        Results
        DThresSwitch
        PixelDependSwitch
    end
    
    methods %initiation
        function obj = MaskPropsCalc(varargin)
            % if nargin < 1
            % listswitch = default;
            obj.PropertyList = obj.getAllList;
            obj.FunctionList = obj.getAllFunList;
            obj.Results = [];
            if isempty(varargin)
                obj.ListSwitch = obj.getDefaultSwitch;
            elseif isnumeric(varargin{1})
                ListSwitch = varargin{1} >0;
                ListSwitch = [ListSwitch, zeros(1, length(obj.getDefaultSwitch)-length(ListSwitch))];
                obj.ListSwitch = ListSwitch;
                %ismember(a,b)
            elseif isstr(varargin{1})||iscell(varargin(1))
                [~, ListSwitch, obj] = obj.MathList(varargin(1));
                obj.ListSwitch = ListSwitch;
            end
        end
    end
    
    methods
        function [PropertyList, obj] = getAllList(obj)
            PropertyList = {'Area','EulerNumber','Orientation','BoundingBox'...
                ,'Extent','Perimeter','Centroid','Extrema','PixelIdxList','ConvexArea'...
                ,'FilledArea','PixelList','ConvexHull','FilledImage','Solidity'...
                ,'ConvexImage','Image','SubarrayIdx','Eccentricity','MajorAxisLength'...
                ,'EquivDiameter','MinorAxisLength','MaxIntensity','MinIntensity'...
            ,'WeightedCentroid','MeanIntensity','PixelValues', 'Circularity', 'RingIntensity',...
            'DistanceToEdge', 'InOutBoundary','VolumeByArea'}';
%             add any additional object properties.
obj.PropertyList = PropertyList;
        end
        
        function [ListSwitch, obj] = getDefaultSwitch(obj)
            ListSwitch = zeros(1,length(obj.getAllList));
            ListSwitch([1, 3, 6, 7]) = 1;
            obj.ListSwitch = ListSwitch;
        end
        
        function [PixelDependSwitch, obj] = getPixelDependSwitch(obj)
            PixelDependSwitch = zeros(1,length(obj.getAllList));
            PixelDependSwitch([1, 10, 11,32]) = 1; % area
            PixelDependSwitch([20, 21, 22, 30]) = 2; % length
            obj.PixelDependSwitch = PixelDependSwitch;
        end
        
        function [DThresSwitch, obj] = getDefaultThresholdableSwitch(obj)
            DThresSwitch = zeros(1,length(obj.getAllList));
            DThresSwitch([1, 2, 3, 5, 6, 10, 11, 15, 19,...
                20, 21, 22, 23, 24, 26, 28, 29, 30, 31, 32]) = 1;
            obj.DThresSwitch = DThresSwitch;
        end
        
        function ToCalcList = getCalcList(obj)
            ToCalcList = {obj.FunctionList{obj.ListSwitch>0}};
        end
        
        function ToCalcList = getNotCalcList(obj)
            ToCalcList = {obj.FunctionList{obj.ListSwitch==0}};
        end
        
        function ToCalcList = getCalcPropsList(obj)
            ToCalcList = {obj.PropertyList{obj.ListSwitch>0}};
        end
        
        function ToCalcList = getNotCalcPropsList(obj)
            ToCalcList = {obj.PropertyList{obj.ListSwitch==0}};
        end
        
        function [PropList, ListSwitch, obj] = MatchLists(obj, PropsNamesCell)
            tocompareOriginal = obj.PropertyList;
            tocompareInList = repmat({PropsNamesCell}, [length(tocompareOriginal),1]);
            ListSwitch = cell2mat(cellfun(@(x,y) sum(strcmp(x, y)), tocompareOriginal, tocompareInList, 'UniformOutput', 0));
            PropList = {obj.PropertyList{ListSwitch>0}};
            obj.ListSwitch = ListSwitch;
        end
        
        function [obj, Results, MinMaxRange] = saveResults(obj, propsCellArray, PixelSize, UnitSwitch)
            if UnitSwitch  == 1
                Multip = PixelSize/(10^6); % in mm
            elseif UnitSwitch == 2
                Multip = PixelSize/1000; % in um;
            elseif UnitSwitch == 3
                Multip = PixelSize; % in nm
            elseif UnitSwitch == 4
                Multip = 1; % in pixel
            end
            Results.propsCellArray = propsCellArray;
            arrayOutIdx = cellfun(@isnumeric, propsCellArray);
            ListRegionProps = obj.getCalcPropsList;
            Results.thresholdableMatrix = [propsCellArray{arrayOutIdx>0}];
            thresM = []; thresP = cell(1,1); k = 1;
            allpropslist = obj.getAllList;
            Pixeldependswitch = obj.getPixelDependSwitch;
            lengthPixel = allpropslist(Pixeldependswitch == 2);
            areaPixel = allpropslist(Pixeldependswitch == 1);
            for i1 = 1:length(arrayOutIdx)
                if arrayOutIdx(i1) >0
                    if isvector(propsCellArray{i1})
                        thresP{k} = ListRegionProps{i1}; %get property name
                        % compare property name
                        % compare length pixel names
                        if sum(strcmp(thresP{k}, lengthPixel))>0
                            propsCellArray{i1} = propsCellArray{i1}*Multip;
                        end
                        % compare area pixel names
                        if sum(strcmp(thresP{k}, areaPixel))>0 
                            propsCellArray{i1} = propsCellArray{i1}*Multip^2;
                        end
                        thresM = [thresM, propsCellArray{i1}];
                        k = k+1;
                    else
                        thresM = [thresM, propsCellArray{i1}];
                        for i2 = 1:size(propsCellArray{i1},2)
                            thresP{k} = [ListRegionProps{i1}, '_Channel_', num2str(i2)]; k = k+1;
                        end
                    end
                end
            end
            Results.thresholdableMatrix = thresM;
            Results.thresholdableProps = thresP;
            if isempty(thresM)
                MinMaxRange = [NaN; NaN];
            else
            MinMaxRange = [min(Results.thresholdableMatrix); max(Results.thresholdableMatrix)];
            end
            obj.Results = [obj.Results Results];
        end
        
        function Results = getResults(obj)
            Results = obj.Results;
        end
       
        
        function [ThresholdableMatrix, ThresholdableProps] = getThresholdableResults(obj)
            if ~isempty(obj.Results)
                ThresholdableMatrix = obj.Results.thresholdableMatrix;
                ThresholdableProps = obj.Results.thresholdableProps;
            else
                ThresholdableMatrix = [];
                ThresholdableProps = [];
            end
        end
        
        function obj = setThresholds(obj, thresPropsNCell, threholdValues)
            % thresPropsNCell is the Cell array of Names
            % thresholdValues is also an Cell array, each containing NX2
            % upper and lower limits.
            % No MaskPropsCalc does not save processed masks in the object.
            if ~isempty(obj.Results)
                obj.getThresholdableResults;
                if ~isempty(obj.Results.thresholdableProps)
                    obj.Thresholds.thresholdNames = thresPropsNCell;
                    obj.Thresholds.thresholdValues = threholdValues;
                end
            end
        end
        
        function [FunctionList, obj] = getAllFunList(obj)
            % add parameter structure in here.
            FunctionList = {@(L)(regionpropsArrayBurrito(L,'Area')),...
                @(L)(regionpropsArrayBurrito(L,'EulerNumber')),...
                @(L)(regionpropsArrayBurrito(L,'Orientation')),...
                @(L)(regionpropsArrayBurrito(L,'BoundingBox')),...
                @(L)(regionpropsArrayBurrito(L,'Extent')),...
                @(L)(regionpropsArrayBurrito(L,'Perimeter')),...
                @(L)(regionpropsArrayBurrito(L,'Centroid')),...
                @(L)(regionpropsArrayBurrito(L,'Extrema')),...
                @(L)(regionpropsArrayBurrito(L,'PixelIdxList')),...
                @(L)(regionpropsArrayBurrito(L,'ConvexArea')),...
                @(L)(regionpropsArrayBurrito(L,'FilledArea')),...
                @(L)(regionpropsArrayBurrito(L,'PixelList')),...
                @(L)(regionpropsArrayBurrito(L,'ConvexHull')),...
                @(L)(regionpropsArrayBurrito(L,'FilledImage')),...
                @(L)(regionpropsArrayBurrito(L,'Solidity')),...
                @(L)(regionpropsArrayBurrito(L,'ConvexImage')),...
                @(L)(regionpropsArrayBurrito(L,'Image')),...
                @(L)(regionpropsArrayBurrito(L,'SubarrayIdx')),...
                @(L)(regionpropsArrayBurrito(L,'Eccentricity')),...
                @(L)(regionpropsArrayBurrito(L,'MajorAxisLength')),...
                @(L)(regionpropsArrayBurrito(L,'EquivDiameter')),...
                @(L)(regionpropsArrayBurrito(L,'MinorAxisLength')),...
                @(L, I)(regionpropsArrayBurrito(L, I,'MaxIntensity')),...
                @(L, I)(regionpropsArrayBurrito(L, I,'MinIntensity')),...
                @(L, I)(regionpropsArrayBurrito(L, I,'WeightedCentroid')),...
                @(L, I)(regionpropsArrayBurrito(L, I,'MeanIntensity')),...
                @(L, I)(regionpropsArrayBurrito(L, I,'PixelValues')),...
                @(L)(circularityCalc(L)), ...
                @(L, I)(ringMeanIntensitiesCalc(L, I)),...
                @(L, I, P)(distanceToEdge(L, I, P)),...
                @(L, I, P)(inOutBoundary(L,I, P)),...
                @(L, I, P)(liquidVolumeByArea(L,I,P))};
            obj.FunctionList = FunctionList;
            %@(L)(maskObjectCircularity(L,16)
        end
    end
end
