classdef MaskPropsCalcFilterPackage < Package
    methods (Access = public)
        function obj = MaskPropsCalcFilterPackage(owner, varargin)
            if nargin == 0
                super_args = {};
            else
                % Check input
                ip =inputParser;
                ip.addRequired('owner',@(x) isa(x,'MovieObject'));
                ip.addOptional('outputDir',owner.outputDirectory_,@ischar);
                ip.parse(owner,varargin{:});
                outputDir = ip.Results.outputDir;
                
                super_args{1} = owner;               
                super_args{2} = [outputDir filesep 'MaskPropsCalcFilterPackage'];
            end
            % Call the superclass constructor
            obj = obj@Package(super_args{:});
        end
        function [status, processExceptions] = sanityCheck(obj,varargin)
            
            % Check that the channels have a psf function
            nProc = length(obj.getProcessClassNames);
            ip = inputParser;
            ip.CaseSensitive = false;
            ip.addRequired('obj');
            ip.addOptional('full',true, @(x) islogical(x));
            ip.addOptional('procID',1:nProc,@(x) all(ismember(x,1:nProc)) || strcmp(x,'all'));
            ip.parse(obj,varargin{:});
            full = ip.Results.full;
            procID = ip.Results.procID;
            if strcmp(procID,'all'), procID = 1:nProc;end
            [status, processExceptions] = sanityCheck@Package(obj,varargin{:});
        end
    end
    methods (Static)
        function name = getName()
            name = 'Mask Properties Calculation and Filtering';
        end
                                 
        function m = getDependencyMatrix(i,j)
            m = [0 0 0 0; % HCS Translocation Scoring process
                 1 0 0 0;
                 1 0 0 0;
                 0 0 1 0];
            if nargin<2, j=1:size(m,2); end
            if nargin<1, i=1:size(m,1); end
            m=m(i,j);
        end
        
        function varargout = GUI(varargin)
            % Start the package GUI
            varargout{1} = MaskPropsCalcFilterPackage(varargin{:});
        end

        function procConstr = getDefaultProcessConstructors(index)
            segProcConstr = {
                @NucSegProcess,...
                @MaskRefinementProcess, ...
                @CalcMaskPropsProcess, ...
                @FilterMaskPropsProcess};
            
            if nargin==0, index=1:numel(segProcConstr); end
            procConstr=segProcConstr(index);
        end
        function classes = getProcessClassNames(index)
            segClasses = {
                'SegmentationProcess',...
                'MaskRefinementProcess', ...
                'CalcMaskPropsProcess', ...
                'FilterMaskPropsProcess'};
            if nargin==0, index=1:numel(segClasses); end
            classes=segClasses(index);
        end
    end
end