function movieData = binningMovieObjects(movieData, varargin)
pString = 'Properties_'; %Prefix for saving masks to file
dName = 'binning_for_channel_';

%% ----------- Input ----------- %%

%Check input
ip = inputParser;
ip.CaseSensitive = false;
ip.addRequired('movieData', @(x) isa(x,'MovieData'));
ip.addOptional('paramsIn',[], @isstruct);
ip.parse(movieData,varargin{:});
paramsIn=ip.Results.paramsIn;
%Get mask binning process and maskcalcprops processes                                                                     
iProc = movieData.getProcessIndex('MaskObjectBinningProcess',1,0);
iProcCal = movieData.getProcessIndex('CalcMaskPropsProcess',1,0);
iProcFilterPaper = movieData.getProcessIndex('FilterPaperSpecSegProcess',1,0);
if ~isempty(iProcFilterPaper)
    FilterPaperMaskProc = movieData.processes_{iProcFilterPaper};
end
if isempty(iProcCal)
    warndlg('CalcMaskPropsProcess has to be performed prior to filtering.');                                                                                                 
end
CalcMPProcess = movieData.processes_{iProcCal};
maskObjectBinningP = movieData.processes_{iProc};
%If the process doesn't exist, create it
if isempty(iProc)
    iProc = numel(movieData.processes_)+1;
    movieData.addProcess(MaskObjectBinningProcess(movieData,...
        movieData.outputDirectory_));                                                                                                 
end

%% set ups

p = parseProcessParams(maskObjectBinningP,paramsIn);

%% load in masks
maskUsed = CalcMPProcess.funParams_.maskUsed;
maskprocessforuse = movieData.processes_{maskUsed};
nChanThresh = length(p.ChannelIndex);
nImages = movieData.nFrames_;
nImTot = nImages * nChanThresh;

%Set up the input and mask directories
inMaskDir = cell(1,nChanThresh);
outMaskDir = cell(1,nChanThresh);
propsNames = cell(1,nChanThresh);
for j = 1:nChanThresh;    
    maskNames(j) = maskprocessforuse.getOutMaskFileNames(p.ChannelIndex(j));
    inMaskDir(j) = maskprocessforuse.outFilePaths_(p.ChannelIndex(j));
    
    %Create string for current directory
    currDir = [p.OutputDirectory filesep dName num2str(p.ChannelIndex(j))];
        
    %Check/create directory
    mkClrDir(currDir);
    %Save this in the process object
    movieData.processes_{iProc}.setOutMaskPath(p.ChannelIndex(j),currDir);
    movieData.processes_{iProc}.setInMaskPath(p.ChannelIndex(j),inMaskDir{j});
    outMaskDir{j} = currDir;
end
 %
selectedProps=CalcMPProcess.funParams_.thresholdableProps{1};
% selectedProps = maskObjectBinningP.funParams_.thresholdableProps{1};
selectedPropsV = maskObjectBinningP.funParams_.selectedPropsV;
% get property name/index
selectedProperty = selectedProps{selectedPropsV};
% get bins
rangeSettingMatrix = maskObjectBinningP.funParams_.rangeSettingMatrix{1};
% matrix compare for each image
% assemble a labeled mask and write it out
% if save a copy
copySavetag = maskObjectBinningP.funParams_.copySavetag;


if feature('ShowFigureWindows')
    wtBar = waitbar(0,['Please wait, computing mask properties for channel ' num2str(p.ChannelIndex(1)) ' ...']);     
else
    wtBar = -1;
end
for iChan = 1:nChanThresh
    disp(['Using masks from ' inMaskDir{iChan} ', storing results in ' outMaskDir{iChan}])
    MPropsC = CalcMPProcess.loadChannelOutput(iChan);
    if copySavetag == 1
        timestampstring = getTimeStampStr;
        mkdir([outMaskDir{iChan} timestampstring]);
    end
    for iImage = 1:nImages;
        if ishandle(wtBar)
            waitbar(((iChan-1)*nImages+iImage) / nImTot,wtBar,['Please wait, filtering properties for channel ' num2str(p.ChannelIndex(iChan)) ' ...']);
        end
        
        %Load the mask for this frame/channel
        currMask = maskprocessforuse.loadChannelOutput(iChan, iImage);%imread([inMaskDir{iChan} filesep maskNames{iChan}{iImage}]);
        % Matrix Compare
        %get matrix
        currMatrix = MPropsC.Results(iImage).thresholdableMatrix;
        %% get upper and lower limits from each bin!!!
        totalNProps = length(selectedProps);
        PropIndex = selectedPropsV;
        rangelimitsCell = getRangeLimits(rangeSettingMatrix, totalNProps, PropIndex); % rangelimitsCell is a cellarray which has multiple upper and lower limits.
        for iBin = 1:length(rangelimitsCell)
            rangelimits = rangelimitsCell{iBin};
            upperlimits = rangelimits(:,2)';
            lowerlimits = rangelimits(:,1)';
            % get logic array
            if isempty(currMatrix)
                break;
            end
            if size(currMatrix, 2) ~= length(lowerlimits)
                errordlg('Intensity Properties have been caculated for multiple channels, please reset the filtering criteria!');
                return;
            end
            [~, AndLogic, ~] = matrixCompare(currMatrix, lowerlimits, upperlimits);
            
            % output mask and imwrite.
            if islogical(currMask);
                currMask = bwlabel(currMask);
            end
            %filteredMask = filterObjects(currMask, AndLogic)>0;
            NumObject{iBin} = find(AndLogic==1);
        end
        if ~isempty(currMatrix)
            binnedMask = uint8(binMasks(currMask, NumObject));
            %write masks and csv
            imwrite(binnedMask,[outMaskDir{iChan} filesep pString maskNames{iChan}{iImage}]);
            objectPairs{iImage} = getObjectPairs(NumObject);
            %if yes_for_copy selected, write a copy with timestamp!
            if copySavetag == 1
                imwrite(binnedMask,[outMaskDir{iChan} timestampstring filesep pString maskNames{iChan}{iImage}]);
            end
            maxIndex = max(binnedMask(:));
            maxIndexCurr = maskObjectBinningP.getMaxIndex;
            if isempty(maxIndexCurr)
                maxIndexCurr = 1;
            end
            if maxIndex <1
                maxIndex = 1;
            end
            maskObjectBinningP.setMaxIndex(max(uint16(maxIndex),uint16(maxIndexCurr)));
            % assemble info to write in csv
            %% first check if there's package for urine detection, if not skip - specific for the p2p project
            % the summaries writing.
            iPackage = movieData.getPackageIndex('MaskQuantPackage',1,0);
            if ~isempty(iPackage)
                % get summaries done!
                if ~isempty(iProcFilterPaper)
                    filterMask = FilterPaperMaskProc.loadChannelOutput(iChan, iImage);
                    summaries(iImage) = assemblePropstoWrite(filterMask, currMask,MPropsC, selectedProperty,...
                        rangeSettingMatrix,iImage, rangelimitsCell,objectPairs{iImage}, movieData,PropIndex);
                    if ~exist('fileIDSummary', 'var')
                        fileIDSummary = [];
                    end
                    fileIDSummary = printSummaryUrineSpots(summaries(iImage), outMaskDir{iChan},fileIDSummary);
                    
                    if copySavetag == 1
                        if ~exist('fileIDSummaryc', 'var')
                            fileIDSummaryc = [];
                        end
                        fileIDSummaryc = printSummaryUrineSpots(summaries(iImage), [outMaskDir{iChan} timestampstring],fileIDSummaryc);
                    end
                    
                end
            end
        else
            imwrite((currMask),[outMaskDir{iChan} filesep pString maskNames{iChan}{iImage}]);
            if copySavetag == 1
                imwrite(currMask,[outMaskDir{iChan} timestampstring filesep pString maskNames{iChan}{iImage}]);
            end
        end
        
    end
    %summaries.imageNumber = nImages;
    wirtePropertyCSVforObjectPairsforUrineSpots(objectPairs, MPropsC, outMaskDir{iChan}, 'mm');
    %printSummaryUrineSpots(summaries, outMaskDir{iChan})
    if copySavetag == 1
        wirtePropertyCSVforObjectPairsforUrineSpots(objectPairs, MPropsC, [outMaskDir{iChan} timestampstring], 'mm');
        %printSummaryUrineSpots(summaries, [outMaskDir{iChan} timestampstring]);
    end
end

if ishandle(wtBar)
    close(wtBar)
end


%% ------ Finalize ------ %%

%save thresholdable filename;

%Store parameters/settings in movieData structure

movieData.processes_{iProc}.setDateTime;
movieData.save; %Save the new movieData to disk


disp('Finished binning objects in the mask!');
