function movieData = calcMovieMaskProperties(movieData, varargin)
% CALCMOVIEMASKPROPERTIES calculates objects in a movie masks
%
% calcMovieMaskProperties
%
% SYNOPSIS calcMovieMaskProperties(movieData, paramsIn)
%
% INPUT
% movieData
% MaskPropsCalc - an object which saves all property and calculation
% functions and options.

pString = 'Properties_'; %Prefix for saving masks to file
dName = 'properties_for_channel_';

%% ----------- Input ----------- %%

%Check input
ip = inputParser;
ip.CaseSensitive = false;
ip.addRequired('movieData', @(x) isa(x,'MovieData'));
ip.addOptional('paramsIn',[], @isstruct);
ip.parse(movieData,varargin{:});
paramsIn=ip.Results.paramsIn;

%Get the indices of any previous speckle detection processes                                                                     
iProc = movieData.getProcessIndex('CalcMaskPropsProcess',1,0);

%get the filter paper mask %% QUICK FIX: SPECIFIC TO URINE DETECTION PROPJECT!!! NEED
%TO FIND SMART WAY TO BLINDLY PASS IN PARAMETERS!!! %%
iProcFilterPaper = movieData.getProcessIndex('FilterPaperSpecSegProcess',1,0);
if ~isempty(iProcFilterPaper)
    FilterPaperMaskProc = movieData.processes_{iProcFilterPaper};
end
% end of THIS QUICK FIX PROCESS FINDING...

%If the process doesn't exist, create it
if isempty(iProc)
    iProc = numel(movieData.processes_)+1;
    movieData.addProcess(CalcMaskPropsProcess(movieData,...
        movieData.outputDirectory_));                                                                                                 
end
subResDetProc = movieData.processes_{iProc};
%Parse input, store in parameter structure
p = parseProcessParams(subResDetProc,paramsIn);
unitswitch = p.saveresultsas; % uinitswitch = 1:nm, 2:um, 3:pixel
unitswitchStr = {'nm', 'um', 'mm','pixel'};
% get props list
propsToCalc =  p.selectedProps;
pixelsize = movieData.pixelSize_;
% get the correct switch
impc = MaskPropsCalc;
[~, switchList, impc] = impc.MatchLists(propsToCalc'); %write impc into disk 
%% load in masks
p.SegProcessIndex = p.MaskProcessIndex;
%p.SegProcessIndex = movieData.getProcessIndex('MaskProcess'); %put this thing into settings.
iP = p.SegProcessIndex;
maskprocessforuse = movieData.processes_{p.SegProcessIndex};
if isfield(maskprocessforuse.funParams_, 'volume_Ul_Per_MMSQ')
    volumePerMMSQ = maskprocessforuse.funParams_.volume_Ul_Per_MMSQ;
end
nChanThresh = length(p.ChannelIndex);
nImages = movieData.nFrames_;
nImTot = nImages * nChanThresh;

%Set up the input and mask directories
inMaskDir = cell(1,nChanThresh);
outMaskDir = cell(1,nChanThresh);
propsNames = cell(1,nChanThresh);
for j = 1:nChanThresh;    
    propsNames(j) = movieData.processes_{iP}.getOutMaskFileNames(p.ChannelIndex(j));
    inMaskDir(j) = movieData.processes_{iP}.outFilePaths_(p.ChannelIndex(j));
    
    %Create string for current directory
    currDir = [p.OutputDirectory filesep dName num2str(p.ChannelIndex(j)), '.mat'];
        
    %Check/create directory
    mkClrDir(p.OutputDirectory);
    %Save this in the process object
    
    movieData.processes_{iProc}.setInImagePath(p.ChannelIndex(j),inMaskDir{j});
    outMaskDir{j} = currDir;
end

%% added step for corrected image loading

%Check the input processes if any and get loader handles for each channel
imLoader = cell(1,nChanThresh);
spi = movieData.getProcessIndex('BackgroundSubtractionProcess');


if ~isempty(spi)
    for i = 1:length(movieData.channels_)
        p.ProcessIndex(i) = spi;
    end
    nChanBackgroundSub = movieData.processes_{spi}.funParams_.ChannelIndex;
else
    for i = 1:length(movieData.channels_)
        p.ProcessIndex(i) = 0;
    end
end

nTotalChan = length(movieData.channels_);

if p.ProcessIndex(j) > 0
    
    for j11 = 1:length(nChanBackgroundSub)
        j1 = nChanBackgroundSub(j11);
        %Check the specified input process
        assert(movieData.processes_{p.ProcessIndex(j1)}.checkChannelOutput(j1),['No valid output for input process specified for channel ' num2str(p.ChannelIndex(j))]);
        imLoader{j1} = @(f)(movieData.processes_{p.ProcessIndex(j1)}.loadChannelOutput(j1,f));
    end
else
    for j = 1:nTotalChan
        %If raw data specified
        imLoader{j} = @(f)(movieData.channels_(j).loadStack(f));
    end
end



%% ---------------- Property Calculations --------------- %%
% if ~p.BatchMode && feature('ShowFigureWindows')
%     wtBar = waitbar(0,['Please wait, computing mask properties for channel ' num2str(p.ChannelIndex(1)) ' ...']);     
% else
    wtBar = -1;
% end       
for iChan = 1:nChanThresh

    disp(['Calculating properties for channel ' num2str(p.ChannelIndex(iChan)) '...']);
    disp(['Using masks from ' inMaskDir{iChan} ', storing results in ' outMaskDir{iChan}])
    
    
    for iImage = 1:nImages;
        if ishandle(wtBar)
            waitbar(((iChan-1)*nImages+iImage) / nImTot,wtBar,['Please wait, calculating properties for channel ' num2str(p.ChannelIndex(iChan)) ' ...']);
        end
        %Load the mask for this frame/channel
        % currMask = maskprocessforuse.loadChannelOutput(iChan, iImage);%imread([inMaskDir{iChan} filesep maskNames{iChan}{iImage}]);
        currMask = maskprocessforuse.loadChannelOutput(p.ChannelIndex(iChan), iImage);
        if ~isempty(iProcFilterPaper)
            % filterMask = FilterPaperMaskProc.loadChannelOutput(iChan, iImage);
            filterMask = FilterPaperMaskProc.loadChannelOutput(p.ChannelIndex(iChan), iImage);
        else
            filterMask = [];
        end
        % build up the additional params input if there's any:
        AdditionalParams.filterMask = filterMask;
        AdditionalParams.distanceMap = bwdist(~imfill(filterMask,'holes'));
        if isfield(maskprocessforuse.funParams_, 'volume_Ul_Per_MMSQ')
            AdditionalParams.volumePerMMSQ = volumePerMMSQ; %% put this into GUI!!!! did it temporarily in previous step.
            % end of building AdditionalParams
        end
        chanforIm = 1:length(movieData.channels_);
        for iImload = chanforIm % by default calculate all avaiable intensity properties
            %currImage(:,:,iImload) = movieData.reader.loadImage(iImload,iImage);
            currImage(:,:,iImload) = imLoader{iImload}(iImage);
        end
        if iImage == 1 
        [MPropsC, MinMaxRange, results(iImage)] = getObjectProperties(currMask, 'PropertyListLogical', switchList,...
            'Image', currImage, 'UnitSwitch', unitswitch, 'PixelSize', pixelsize, 'AdditionalParams', AdditionalParams);
        else
            [MPropsC, MinMaxRange, results(iImage)] = getObjectProperties(currMask, 'PropertyListLogical', switchList,...
                'Props', MPropsC,'Image', currImage, 'UnitSwitch', unitswitch, 'PixelSize', pixelsize,'AdditionalParams', AdditionalParams);
        end
        newMinMaxRange = zeros(size(MinMaxRange));
        newMinMaxRange(1,:) = MinMaxRange(1,:);
        newMinMaxRange(1,:) = min([MinMaxRange(1,:);newMinMaxRange(1,:)]);
        newMinMaxRange(2,:) = max([MinMaxRange(2,:);newMinMaxRange(2,:)]);
        %save([outMaskDir{iChan} filesep pString propsNames{iChan}{iImage} '.mat'], 'MPropsC');
        %write alll thresholdable properties
        if islogical(currMask);
            currMask = bwlabel(currMask);
        end
        objectNumbers = unique(currMask);
        objectNumbers = objectNumbers(objectNumbers>0);
        filePath = outMaskDir{iChan}(1:end-4);
        if iImage == 1
            fileID = wirtePropertyCSVforObjects(iImage, objectNumbers, MPropsC, filePath, ['_', unitswitchStr{unitswitch},'_'], []);
        else
            fileID = wirtePropertyCSVforObjects(iImage, objectNumbers, MPropsC, filePath, ['_', unitswitchStr{unitswitch},'_'], fileID);
        end
    end
    save(outMaskDir{iChan}, 'MPropsC');
    movieData.processes_{iProc}.setOutFilePath(iChan,outMaskDir{iChan});
    %save range
    csvwrite([p.OutputDirectory filesep 'ThresholdablePropsRange.csv'], newMinMaxRange);
    opara = movieData.processes_{iProc}.getParameters;
    opara.thresholdableProps{iChan} = results(iImage).thresholdableProps';
    opara.maskUsed = p.SegProcessIndex;
    opara.newMinMaxRange{iChan} = newMinMaxRange';
    
end

if ishandle(wtBar)
    close(wtBar)
end


%% ------ Finalize ------ %%

%save thresholdable filename;

%Store parameters/settings in movieData structure

movieData.processes_{iProc}.setDateTime;
movieData.processes_{iProc}.setParameters(opara);
movieData.save; %Save the new movieData to disk

disp('Finished refining masks!')









