function [Props, MinMaxRange,Results] = getObjectProperties(labeledMask, varargin)
%GETMASKOBJECTPROPERTIES is a generic/abstract function taking in a labeled Mask, and
%outputs a cellarray of object properties. This function does not contain
%any actual calculation function but takes in and output an object
%containing the set of items to be run on the input mask.

%

%% check input
if islogical(labeledMask);
    labeledMask = bwlabel(labeledMask);
end

if nargin == 1
    Props = MaskPropsCalc; % initiate a props list object and get default list
    ListRegionProps = Props.getCalcList;
else
    ip = inputParser;
    ip.addOptional('Props', [], @(x) isa(x, 'MaskPropsCalc'));
    ip.addOptional('PropertyListLogical', @isnumeric);
    ip.addOptional('Image', []);
    ip.addOptional('UnitSwitch', 1);
    ip.addOptional('PixelSize', 1);
    ip.addOptional('AdditionalParams',[]);
    %ip.addOptional('PropertyListName', @iscell);
    ip.parse(varargin{:});
    if ~isempty(ip.Results.Props)
        Props = ip.Results.Props;
    else
    Props = MaskPropsCalc(ip.Results.PropertyListLogical);
    end
    %if length(ip.Results.PropertyListLogical)>26
    ListRegionProps = Props.getCalcList;
    UnitSwitch = ip.Results.UnitSwitch;
    PixelSize = ip.Results.PixelSize;
    if isempty(PixelSize)
        PixelSize = 1;
    end
end
Images = ip.Results.Image;
AdditionalParams = ip.Results.AdditionalParams;
%% process all listed properties
% regionprops_properties
if max(labeledMask(:)) == 0
    propertyCellArray = cell(1, length(ListRegionProps));
else
    for i = 1:length(ListRegionProps)
        functionHandle = ListRegionProps{i};
        % if the function only needs one input
        if nargin(functionHandle) == 1
            propertyCellArray{i} = functionHandle(labeledMask);
            % if the function needs mask and intensity(image) as inputs, will
            % loop through all channels.
        elseif nargin(functionHandle) == 2
            if size(Images, 3) >= 1
                if isnumeric(functionHandle(labeledMask, Images(:,:,1)))
                    for iIm = 1:size(Images, 3)
                        
                        propsOut(:,iIm) = functionHandle(labeledMask, Images(:,:,iIm));
                    end
                else
                    for iIm = 1:size(Images, 3)
                        propsOuttemp{iIm} = functionHandle(labeledMask, Images(:,:,iIm));
                    end
                    
                    for iImR = 1:length(propsOuttemp)
                        for iImL = 1:length(propsOuttemp{1})
                            propsOut{iImL, iImR} = propsOuttemp{iImR}{iImL};
                        end
                    end
                end
                propertyCellArray{i} = propsOut;propsOut = [];
            end
            %labeledMaskCell{i} = labeledMask;
            %% else... needs to parse out possibilities for functions needing additional inputs, such as extra mask, or numerical input.!!
        elseif nargin(functionHandle) > 2  % no multichannel support available here!
            propertyCellArray{i} = functionHandle(labeledMask, Images(:,:,1), AdditionalParams);
        end
        
        % %propertyCellArray = cellfun(@regionpropsArrayBurrito, labeledMaskCell, ListRegionProps, 'UniformOutput', false);
        % arrayOutIdx = cellfun(@isnumeric, propertyCellArray);
        % thresholdableMatrix = [propertyCellArray{arrayOutIdx>0}];
        % thresholdableProps = {ListRegionProps{arrayOutIdx>0}}; %put this into MaskPropsCal property
    end
end
[Props, Results, MinMaxRange] = Props.saveResults(propertyCellArray, PixelSize, UnitSwitch);

% other properties