function movieData = pairingMovieObjects(movieData, varargin)
pString = 'Properties_'; %Prefix for saving masks to file
dName = 'properties_for_channel_';

%% ----------- Input ----------- %%

%Check input
ip = inputParser;
ip.CaseSensitive = false;
ip.addRequired('movieData', @(x) isa(x,'MovieData'));
ip.addOptional('paramsIn',[], @isstruct);
ip.parse(movieData,varargin{:});
paramsIn=ip.Results.paramsIn;
%Get the indices of any previous speckle detection processes                                                                     
iProc = movieData.getProcessIndex('MaskObjectPairingProcess',1,0);
%If the process doesn't exist, create it
if isempty(iProc)
    iProc = numel(movieData.processes_)+1;
    movieData.addProcess(MaskObjectPairingProcess(movieData,...
        movieData.outputDirectory_));                                                                                                 
end
maskObjectParingP = movieData.processes_{iProc};
disthres = maskObjectParingP.funParams_.distanceThreshold;
methodInd = maskObjectParingP.funParams_.methodInd;
%Parse input, store in parameter structure
p = parseProcessParams(maskObjectParingP,paramsIn);
p.MaskPropsProcess = movieData.getProcessIndex('CalcMaskPropsProcess');
p.OriginalMaskProcess = movieData.getProcessIndex('NucSegProcess');
p.NucleiProcess = movieData.getProcessIndex('NucleiFilterMaskPropsProcess'); %%%put this thing into settings.%%%%
p.MicroNucleiProcess = movieData.getProcessIndex('MNucleiFilterMaskPropsProcess'); %%%put this thing into settings.%%%
iPC = p.MaskPropsProcess; CalcMPProcess = movieData.processes_{iPC};
iPN = p.NucleiProcess; nucleiProcess = movieData.processes_{iPN};
iPM = p.MicroNucleiProcess; micronucleiProcess = movieData.processes_{iPM};
iPO = p.OriginalMaskProcess; originalmaskProcess = movieData.processes_{iPO};
% get the area threshold from nucleifiltering step
paramNames = nucleiProcess.funParams_.thresholdableProps{1};
paramInd = strcmp('Area', paramNames)>0;

% check both micronuc upper (1st) and parentnuc lower (2nd)
% threshold whichever lower. if min(th1, th2) is -Inf pop a
% warning, and pass through.
% get the lower threshold of parent nulcei area.
areaThres = nucleiProcess.funParams_.rangeSettingMatrix{1}(paramInd,1);
% get the upper threshold of micro nuclei area.
areaThresMicro =  micronucleiProcess.funParams_.rangeSettingMatrix{1}(paramInd,2);
% get the lower threshold between areaThres&areaThresMicro
areaThres = min(areaThres, areaThresMicro);
% pop a warning if areaThres is -Inf
if areaThres == -Inf
    warning('No area threshold is being used in getting parent nuclei mask, Mask is (WholeMask - Micro Nuclei Mask)');
end
% set output directories
%nChanThresh = length(p.ChannelIndex);
nChanThresh = 1;
nImages = movieData.nFrames_;
nImTot = nImages * nChanThresh;

%Set up the input and mask directories
inMaskDir = cell(1,nChanThresh);
outMaskDir = cell(1,nChanThresh);
for j = 1:nChanThresh;    
    maskNames(j) = nucleiProcess.getOutMaskFileNames(p.ChannelIndex(j));
        
    %Create string for current directory
    currDir = [p.OutputDirectory filesep dName num2str(p.ChannelIndex(j))];
        
    %Check/create directory
    mkClrDir(currDir);
    %Save this in the process object
    movieData.processes_{iProc}.setOutMaskPath(p.ChannelIndex(j),currDir);
    %movieData.processes_{iProc}.setInMaskPath(p.ChannelIndex(j),inMaskDir{j});
    % ?? multiple path inputs....
    outMaskDir{j} = currDir;
end



%% ---------------- Pairing Calculations --------------- %%
if feature('ShowFigureWindows')
    wtBar = waitbar(0,['Please wait, pairing mask objects for channel ' num2str(p.ChannelIndex(1)) ' ...']);
else
    wtBar = -1;
end
for iChan = 1:nChanThresh
    MPropsC = CalcMPProcess.loadChannelOutput(iChan);
    
    for iImage = 1:nImages;
        if ishandle(wtBar)
            waitbar(((iChan-1)*nImages+iImage) / nImTot,wtBar,['Please wait, filtering properties for channel ' num2str(p.ChannelIndex(iChan)) ' ...']);
        end
        %Load the mask for this frame/channel
        FilteredMaskNuc = nucleiProcess.loadChannelOutput(iChan, iImage);
        %new way:keeping unfilted non-micronuclei mask
        wholeMask = originalmaskProcess.loadChannelOutput(iChan, iImage);
        % get nuclei object numbering
        currMaskMicroMuc = micronucleiProcess.loadChannelOutput(iChan, iImage);
        %wholeMask = currMaskNuc+uint8(currMaskMicroMuc);
        whMaskName = class(wholeMask);
        currMaskNuc = wholeMask - cast(currMaskMicroMuc,whMaskName);
        %remove small objects according to user's settings.
        %className = class(currMaskNuc);
        % use regionprops to get rid of smaller objects.
        ObjectAreas = regionprops(currMaskNuc, 'Area');
        ObjectAreasArray = double([ObjectAreas.Area]');
        passedObjectLogicArray = ObjectAreasArray > areaThres;
        currMaskNuc = filterObjects(currMaskNuc, passedObjectLogicArray);
        %currMaskNuc = cast(bwareaopen(currMaskNuc, areaThres), className).*currMaskNuc;
        NucleiNum = unique(currMaskNuc(:));
        NucleiNum = NucleiNum(NucleiNum>0); % get rid of 0 backgrounds
        % get micro nuclei object numbering
        MicroNucleiNum = unique(currMaskMicroMuc(:));
        MicroNucleiNum = MicroNucleiNum(MicroNucleiNum>0); % get rid of 0 backgrounds
        
        %% if pairing by different criteria, differentiate from here! or write a wrapper.
        %get matrix
        propsNameforPairing = 'Centroid';
        allProps = MPropsC.getCalcPropsList;
        numofProp = strcmp(allProps, propsNameforPairing);
        currPosition = MPropsC.Results(iImage).propsCellArray{numofProp>0};
        if ~isempty(NucleiNum)
            if methodInd == 1
                %% pairing by distance transform
                objectPairs{iChan}{iImage} = pairObjectbyDistanceTransform(currMaskNuc, currPosition, NucleiNum, MicroNucleiNum, disthres);
            elseif methodInd == 2
                %% pairing by centroid position
                objectPairs{iChan}{iImage} = pairObjectsbyDistance(NucleiNum, MicroNucleiNum, currPosition, disthres);
            end
            FilteredNucleiNum = unique(FilteredMaskNuc(:));
            FilteredNucleiNum = FilteredNucleiNum(FilteredNucleiNum>0);
            %% separate good ones from the whole
            [goodobjectPairs{iChan}{iImage}, badobjectPairs{iChan}{iImage},goodobjectPairsMicroNucPairs{iChan}{iImage}] = separateObjectPairs(objectPairs{iChan}{iImage}, FilteredNucleiNum);
            %% mask output writing
            pairPropsCSVfilePath = outMaskDir{iChan};
            %wirtePropertyCSVforObjectPairs(objectPairs{iChan}, MPropsC, pairPropsCSVfilePath);
            PairedMask = makeGroupedMask(wholeMask,goodobjectPairs{iChan}{iImage});
            PairedMask = cast(PairedMask,minIntClass(max(PairedMask(:))));
            
        else
            PairedMask = zeros(size(wholeMask));
        end
        imwrite(PairedMask,[outMaskDir{iChan} filesep pString maskNames{iChan}{iImage}]);
        maxIndex = max(PairedMask(:));
        maxIndexCurr = maskObjectParingP.getMaxIndex;
        if isempty(maxIndexCurr)
            maxIndexCurr = 1;
        end
        if maxIndex <1
            maxIndex = 1;
        end
        maskObjectParingP.setMaxIndex(max(uint16(maxIndex),uint16(maxIndexCurr)));
        
    end
    %save limits range
    %save([outMaskDir{j}, filesep, 'objectPairing.mat'],objectPairs);
    if ~isempty(movieData.pixelSize_)
        pixelsize = movieData.pixelSize_;
    else
        pixelsize = 1;
    end
    wirtePropertyCSVforObjectPairs(goodobjectPairs{iChan}, MPropsC, pairPropsCSVfilePath,'Filtered_');
    if ~isempty(badobjectPairs{iChan})
        wirtePropertyCSVforObjectPairs(badobjectPairs{iChan}, MPropsC, pairPropsCSVfilePath,'LeftOver_');
    end
    if ~isempty(goodobjectPairsMicroNucPairs{iChan})
        wirtePropertyCSVforObjectPairs(goodobjectPairsMicroNucPairs{iChan}, MPropsC, pairPropsCSVfilePath,'Filtered_with_Micronuclei_Only_');
    end
end

if ishandle(wtBar)
    close(wtBar)
end


%% ------ Finalize ------ %%

%save thresholdable filename;

%Store parameters/settings in movieData structure

movieData.processes_{iProc}.setDateTime;
movieData.save; %Save the new movieData to disk

disp('Finished pairing objects in the mask!');
